Attribute VB_Name = "BRISCOLA1"
Option Explicit
                                    
'
' Application Program Information and related constants
'
Global Const App_Version$ = "Version 1.0, 10 April 1996"
Global Const App_Copyright$ = "Copyright  1996 Sensible Software"
Global Const App_Author$ = "Written by A.Zanna"

Global Const SEC_GLOBAL = "Briscola"
Global Const KEY_WORKDIR = "WorkDir"

Global Const App_DDEShare = "Briscola$"
Global Const App_FileType = "Briscola Game"
Global Const App_FileExt = "bri"
Global Const App_ClipFormat = 1             ' Text. We could check for more than one format...

Global Const App_Profile = "Briscola.INI"

Global App_Debug

Global Game_OtherPC As String               ' name of other workstation
Global Game_OtherPlayer As String           ' name of other guy

Global Game_InProgress As Integer           ' Flag a game is in progress
Global Game_FileName$                       ' Current file name
Global Game_Mode As Integer                 ' normal network or demo

Global Game_BriscolaSuit  As Integer

Global Hand_Number As Integer               ' # of hand being played
Global Hand_Winner As Integer               ' winner of this hand (<> 0 flags Hand_Next() needs running)
Global Hand_CardPlayer1  As Integer         ' cards played by player 1 and 2
Global Hand_CardPlayer2  As Integer
Global Hand_PlayerTurn  As Integer          ' # of player who's to deal or play

Global Player1_Score  As Integer            ' score for player 1
Global Player1_AutoPlay As Integer          ' autoplay enabled for player 1
Global Player1_Name As String               ' cached player Name

Global Player2_AutoPlay As Integer          ' autoplay enabled for player 2
Global Player2_Score  As Integer            ' score for player 2
Global Player2_Name As String               ' cached player Name

Global Const Game_NoFileName$ = "Unnamed"   ' Used when no file active

Global Const MUST_PLAY = "'s turn"
Global Const WINS_HAND = " wins hand"
Global Const WINS_GAME = " wins game! "
Global Const GAME_IS_DRAW = "Game is a draw"
Global Const NOT_YOUR_TURN = ", it's not your turn!"
Global Const NOT_YOUR_CARDS = ", those are not your cards!"
Global Const NOT_NOW = ", you can't do that just now"

Global Const MSG_MODE_NORMAL = "Mode: Normal"
Global Const MSG_MODE_NETWORK = "Mode: Network"
Global Const MSG_MODE_DEMO = "Mode: Demo"

Global Const MODE_NORMAL = 1
Global Const MODE_NETWORK = 2
Global Const MODE_DEMO = 3

Function CardDesc$(c As Integer)
    Dim S%, V%, D$

    S% = CardSuit(c)
    V% = CardValue(c)

    Select Case V%
        Case 1 To 9
            D$ = Format$(V%)
        Case JACK
            D$ = "J"
        Case QUEEN
            D$ = "Q"
        Case KING
            D$ = "K"
        Case JOKER
            D$ = "*"
        Case Else
            D$ = "?"
    End Select
    
    D$ = D$ & " of "

    Select Case S%
        Case HEARTS
            D$ = D$ & "HRT"
        Case SPADES
            D$ = D$ & "SPD"
        Case CLUBS
            D$ = D$ & "FLW"
        Case DIAMONDS
            D$ = D$ & "DMN"
        Case Else
            D$ = D$ & "???"
    End Select

    CardDesc$ = D$

End Function

'
' Find card that's higher than given card (lowest first).
' same suit or any, depending on pattern
'
' retuns card index
'
Function CardPack_FindHigherThan(cs As Control, pat%) As Integer
    Dim S%, V%, I%, c%

    c% = 0
    S% = CardSuit(pat%)
    V% = CardValue(pat%)

    For I% = V% + 1 To KING
        c% = cs.Find(S% + I%)
        If c% <> CARD_NONE Then Exit For
    Next I%

    CardPack_FindHigherThan = c%

End Function

'
' Find card that's lower than given card (highest first).
' same suit or any, depending on pattern
'
' retuns card index
'
Function CardPack_FindLowerThan(cs As Control, pat%) As Integer
        Dim S%, V%, I%, c%

        c% = 0
        S% = CardSuit(pat%)
        V% = CardValue(pat%)

        For I% = V% - 1 To ACE Step -1
            c% = cs.Find(S% + I%)
            If c% <> CARD_NONE Then Exit For
        Next I%

        CardPack_FindLowerThan = c%

End Function

Function Game_AllowAbort()
    
    Dim response As Integer
    Dim Msg As String

    response = IDNO
    Game_AllowAbort = True

    If Game_InProgress = True Then
        Msg = "A game is in progress" & Chr$(10)
        Msg = Msg & "Save it before continuing?"
        response = MsgBox(Msg, MB_YESNOCANCEL + MB_ICONQUESTION, App.Title)
    End If

    Select Case response
        Case IDYES
            Game_Save Game_FileName$
        Case IDNO
        Case IDCANCEL
            Game_AllowAbort = False
    End Select
End Function

'
' Game plays for a player if:
' - it's this player's turn
' - autoplay is enabled for this player
' - 3 cards have been given to this player (or it's last hand)
'
Sub Game_AutoPlay()
    
    ' up to player one
    If Hand_PlayerTurn = 1 And Player1_AutoPlay Then
        If Game.Player1.NumCards = 3 Or Game.Deck.NumCards = 0 Then
            Game.OnTable.TopCard = Game_PlayCard(Game.Player1)
        End If
    End If

    ' up to player two
    If Hand_PlayerTurn <> 1 And Player2_AutoPlay Then
        If Game.Player2.NumCards = 3 Or Game.Deck.NumCards = 0 Then
            Game.OnTable.TopCard = Game_PlayCard(Game.Player2)
        End If
    End If

End Sub

Function Game_CalcScore(Deck As Control) As Integer
    Dim Score%

    ' aces value = 11
    Score% = 11 * Deck.Count(ACE)
    
    ' 3 value = 10
    Score% = Score% + 10 * Deck.Count(3)

    ' king, queen, jack = 3,2,1
    Score% = Score% + 4 * Deck.Count(KING)
    Score% = Score% + 3 * Deck.Count(QUEEN)
    Score% = Score% + 2 * Deck.Count(JACK)

    Game_CalcScore = Score%

End Function

Sub Game_Clear()
    
    Game_InProgress = False
    Game.GameTimer.Enabled = False

    Game_FileName$ = Game_NoFileName$
    Table_Clear

    Player1_Score = 0
    Player2_Score = 0
    
    Game_ShowScore

End Sub

Function Game_Connect(OtherPC As String, IsReply As Integer) As Integer
    Dim S$

    On Error GoTo net_err

    Game_Connect = False

    'If OtherPC = "" Then Exit Function
    'Debug.Print "----- Connecting -----"
    
    ' give time to other replay to accept challenge (20s)
    If IsReply Then
        Game.Player2Name.LinkTimeout = 50
    Else
        Game.Player2Name.LinkTimeout = 200
    End If

    ' This is used to get a link on which a challenge is sent
    NDDEConnect Game.Player2Name, OtherPC, App_DDEShare, "Player1Name"

    
    If Not IsReply Then
        ' if we're connecting (NOT replying to a connection)
        ' send a challenge, with our computer name as a command
        S$ = NetHostName$() & "," & Player1_Name
        Game_NetCommand "C", S$
    End If

    ' these are cross-linked
    NDDEConnect Game.Player1, OtherPC, App_DDEShare, "Player2"
    NDDEConnect Game.Player2, OtherPC, App_DDEShare, "Player1"

    NDDEConnect Game.Stack1, OtherPC, App_DDEShare, "Stack2"
    NDDEConnect Game.Stack2, OtherPC, App_DDEShare, "Stack1"
    
    NDDEConnect Game.OnTable, OtherPC, App_DDEShare, "OnTable"
    NDDEConnect Game.Deck, OtherPC, App_DDEShare, "Deck"
    NDDEConnect Game.BRISCOLA, OtherPC, App_DDEShare, "Briscola"
    
    Game_Connect = True
    'Debug.Print "----- Connected OK -----"

    Exit Function

net_err:
    
    Game_Disconnect
    MsgBox "Failed Connecting to " & OtherPC & Chr$(10) & " (" & Error$ & ")"
    Exit Function

End Function

Sub Game_ConnectBack()
    If Game_Connect(Game_OtherPC, True) Then
        Game_NextPlayerIs 1
        Game_Start
    Else
        Game_Disconnect
        Game_ModeNormal
    End If

End Sub

'
' Returns # of cards already played matching pattern
' for strategy support.
'
' Simulates memory by peeking into the 2 players
' 'captured cards' stacks
'
Function Game_CountPlayed(c%) As Integer

    Dim n%

    n% = Game.Stack1.Count(c%)
    n% = n% + Game.Stack2.Count(c%)

    Game_CountPlayed = c%

End Function

Sub Game_Disconnect()

    Game.Player2Name.LinkMode = 0

    Game.Player1.LinkMode = 0
    Game.Player2.LinkMode = 0

    Game.Stack1.LinkMode = 0
    Game.Stack2.LinkMode = 0

    Game.OnTable.LinkMode = 0
    Game.Deck.LinkMode = 0
    Game.BRISCOLA.LinkMode = 0

End Sub

Sub Game_Finish(fAbort As Integer)
    
    If Game_InProgress Then
        
        If fAbort Then
            If Not Game_AllowAbort() Then Exit Sub

            If Game_Mode = MODE_NETWORK Then
                Game_Disconnect
            End If
            Game_ModeNormal
        Else
            If Player1_Score > Player2_Score Then
                MsgBox Game_PlayerName(1) & WINS_GAME & Player1_Score & "-" & Player2_Score
            Else
                If Player1_Score < Player2_Score Then
                    MsgBox Game_PlayerName(2) & WINS_GAME & Player2_Score & "-" & Player1_Score
                Else
                    MsgBox GAME_IS_DRAW
                End If
            End If
        End If
        
        Game.GameTimer.Enabled = False
        Game_InProgress = False

    End If
End Sub

Sub Game_GiveHint()
    Dim c%

    If Hand_PlayerTurn = 1 Then
        c% = Game_PlayCard(Game.Player1)
        Hand_CardPlayer1 = c%

        If c% <> CARD_EMPTY Then
            Game.OnTable.TopCard = c%
        End If
    Else
        Game.MessageView = Game_PlayerName(1) & NOT_YOUR_TURN

    End If

End Sub

Sub Game_Listen()
    Dim r%

    r% = NDDEListen(App_DDEShare, "BRISCOLA", "Table")

End Sub

Sub Game_ModeDemo()
    
    Game.Player2Name = "Computer 2"
    Game.Player1Name = "Computer 1"
        
    Player1_AutoPlay = True
    Player2_AutoPlay = True

    Game.OptPeek.Checked = 1
    Game.Player2.StackFacing = CARDS_FACING_UP
    
    Game.Mode = MSG_MODE_DEMO
    Game_Mode = MODE_DEMO
    
End Sub

Sub Game_ModeNetwork()
    
    Game.Player1Name = Player1_Name
    
    If App_Debug Then
        Player1_AutoPlay = True
    Else
        Player1_AutoPlay = False
    End If

    Player2_AutoPlay = False

    If App_Debug Then
        Game.OptPeek.Checked = 1
        Game.Player2.StackFacing = CARDS_FACING_UP
        Game.GameTimer.Interval = 100   ' faster than normal
    Else
        Game.OptPeek.Checked = 0
        Game.Player2.StackFacing = CARDS_FACING_DOWN
    End If

    Game_Mode = MODE_NETWORK
    Game.Mode = MSG_MODE_NETWORK
    
End Sub

Sub Game_ModeNormal()
    
    Game.Player2Name = "Computer"
    Game.Player1Name = Player1_Name

    Player1_AutoPlay = False
    Player2_AutoPlay = True

    Game.OptPeek.Checked = 0
    Game.Player2.StackFacing = CARDS_FACING_DOWN

    Game.Mode = MSG_MODE_NORMAL
    Game_Mode = MODE_NORMAL

End Sub

Sub Game_Msg(Msg$)
    Game.MessageView.Caption = Msg$
End Sub

'
' Manda comando al partner, con parametri
'
Sub Game_NetCommand(c$, p$)
    Game.Player2Name.LinkExecute c$ & p$
End Sub

Sub Game_New()
    
    If Game_AllowAbort() = True Then
        
        Game_Clear
        Game_Start
        
        Hand_DealFirst
        Hand_Clear

    End If

End Sub

Sub Game_NextPlayer()
    
    Game_NextPlayerIs (Hand_PlayerTurn + 1)

End Sub

Sub Game_NextPlayerIs(pl%)
    
    Hand_PlayerTurn = pl% Mod 2
    Game.MessageView = Game_PlayerName(Hand_PlayerTurn) & MUST_PLAY

End Sub

Sub Game_Open(fname As String)
    
    Dim S$

    On Error GoTo open_error

    Open fname For Random As #1

    On Error GoTo open_done
    
    Get #1, , S$
    Game.Player1 = S$
    Get #1, , S$
    Game.Player2 = S$
    Get #1, , S$
    Game.Stack1 = S$
    Get #1, , S$
    Game.Stack2 = S$
    Get #1, , S$
    Game.Deck = S$
    Get #1, , S$
    Game.BRISCOLA = S$
    Get #1, , S$
    Game.OnTable = S$

    Get #1, , Hand_CardPlayer1
    Get #1, , Hand_CardPlayer2
    Get #1, , Hand_PlayerTurn
    Get #1, , Game_BriscolaSuit

    Game_FileName$ = fname
    Form_SetTitle Game

open_done:

    Close #1
    Exit Sub

open_error:
    ReportError "Can't open File" & Chr$(10) & "'" & fname & "'"
    Exit Sub
End Sub

Function Game_Options() As Integer
    Dim New_mode As Integer
    
    Options.Show 1
    
    If Options.Tag = "OK" Then
        Player1_Name = Options.NameBox
    
        If Options.OptVsNetwork.Value Then
            New_mode = MODE_NETWORK
            Else
                If Options.OptDemo.Value Then
                    New_mode = MODE_DEMO
                Else
                    New_mode = MODE_NORMAL
            End If
        End If
    
    
        ' if different, change mode and start new game
        
        If New_mode <> Game_Mode Then
            Select Case New_mode
                Case MODE_NETWORK
                    
                    If Game_Connect(NetBrowseHost$(), False) Then
                        Game_ModeNetwork
                        Game_NextPlayerIs 2  ' let other player start
                        Game_New             ' init our decks
                    End If
                
                Case MODE_DEMO
                    Game_ModeDemo
                    Game_New
                
                Case Else
                    Game_ModeNormal
                    Game_New
            End Select
            
        End If

        Game_Options = True
    Else
        Game_Options = False
    End If

End Function

'
' Implements simple player game strategy
'
Function Game_PlayCard(pl As Control) As Integer
    
    Dim c%, t_val%, t_suit%, I%

    c% = CARD_NONE
    
    Table_Disable "Thinking..."
    
    ' must play against table
    If Game.OnTable.NumCards = 1 Then
        t_val% = Game.OnTable.Value(0)
        t_suit% = Game.OnTable.Suit(0)
        
        ' no points on the table. Take it only if
        ' we can make some points (jack, queen)
        If t_val% = 2 Or (t_val% > 3 And t_val% < JACK) Then
            
            c% = pl.Find(JACK + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen
        
            c% = pl.Find(QUEEN + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen
        
        End If

        ' small points on the table. Try to take it.
        If t_val% >= JACK And t_val% < KING Then
            
            c% = CardPack_FindHigherThan(pl, t_val% + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen

            ' may also want to use 3 or ACE if deck is running low
            If Game.Deck.NumCards < 20 Then
                c% = pl.Find(3 + t_suit%)
                If c% <> CARD_NONE Then GoTo Chosen
            
                c% = pl.Find(ACE + t_suit%)
                If c% <> CARD_NONE Then GoTo Chosen
            End If
        End If
        
        ' always take king, try with 3 or ace
        If t_val% = KING Then
            c% = pl.Find(3 + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen
            
            c% = pl.Find(ACE + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen
        End If
    
        ' always take 3, can only be taken by ace
        If t_val% = 3 Then
            c% = pl.Find(ACE + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen
        End If


        ' we want that 3 or ace, but can't beat it with the same suit
        ' try taking it with a briscola, lowest first (3 and ace last)
        If t_val% = 3 Or t_val% = ACE Then
            c% = pl.Find(2 + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen
            
            c% = CardPack_FindHigherThan(pl, 3 + Game_BriscolaSuit%)
            If c% <> CARD_NONE Then GoTo Chosen

            c% = pl.Find(3 + Game_BriscolaSuit%)
            If c% <> CARD_NONE Then GoTo Chosen
        
            c% = pl.Find(ACE + Game_BriscolaSuit%)
            If c% <> CARD_NONE Then GoTo Chosen
        
        End If

        ' card on table is nil points, but maybe we can make points
        ' taking it with a higher card of ours (not a briscola)
        
        If t_suit% <> Game_BriscolaSuit Then
            ' use ace only if 3 is gone
            If Game_CountPlayed(3 + t_suit%) > 0 Then
                c% = pl.Find(ACE + t_suit%)
                If c% <> CARD_NONE Then GoTo Chosen
            End If
    
            ' use 3 only if king is gone
            If Game_CountPlayed(KING + t_suit%) > 0 Then
                c% = pl.Find(3 + t_suit%)
                If c% <> CARD_NONE Then GoTo Chosen
            End If
                
            ' use king only if queen is gone
            If Game_CountPlayed(QUEEN + t_suit%) > 0 Then
                c% = pl.Find(KING + t_suit%)
                If c% <> CARD_NONE Then GoTo Chosen
            End If
            
            ' use queen only if jack is gone
            If Game_CountPlayed(JACK + t_suit%) > 0 Then
                c% = pl.Find(QUEEN + t_suit%)
                If c% <> CARD_NONE Then GoTo Chosen
            End If
    
            c% = pl.Find(JACK + t_suit%)
            If c% <> CARD_NONE Then GoTo Chosen
        End If

    End If

    ' if we get here, we are either playing 1st
    ' or we're not interested in the card that's on the table

    ' find lowest card in hand that's not a briscola
    For I% = HEARTS To SPADES Step ONE_SUIT
        
        If I% <> Game_BriscolaSuit% Then
            ' see if we have a 2
            c% = pl.Find(I% + 2)
            If c% <> CARD_NONE Then GoTo Chosen
            
            ' or any lowest card that's not points
            c% = CardPack_FindHigherThan(pl, I% + 3)
            If c% <> CARD_NONE Then
                If pl.Value(c%) < JACK Then GoTo Chosen
            End If
        End If
    Next I%

    
    ' Next, see if we can play a briscola that's not points
    ' see if we have a 2
    c% = pl.Find(Game_BriscolaSuit% + 2)
    If c% <> CARD_NONE Then GoTo Chosen
    
    ' or any lowest briscola that's also not points
    c% = CardPack_FindHigherThan(pl, Game_BriscolaSuit% + 3)
    If c% <> CARD_NONE Then
        If pl.Value(c%) < JACK Then GoTo Chosen
    End If
    

    ' Next, see if we have to give in small points (not briscola)
    For I% = HEARTS To SPADES Step ONE_SUIT
        
        If I% <> Game_BriscolaSuit% Then
        ' look for Jack, Queen, King
            c% = CardPack_FindHigherThan(pl, I% + 10)
            If c% <> CARD_NONE Then GoTo Chosen
        End If
    Next I%
    

    ' Next, see if we have to play a briscola that's small points
    ' look for Jack, Queen, King
    c% = CardPack_FindHigherThan(pl, Game_BriscolaSuit% + 10)
    If c% <> CARD_NONE Then GoTo Chosen

    
    ' Next, see if we have to play a briscola that's BIG points
    c% = pl.Find(Game_BriscolaSuit% + 3)
    If c% <> CARD_NONE Then GoTo Chosen

    c% = pl.Find(Game_BriscolaSuit% + ACE)
    If c% <> CARD_NONE Then GoTo Chosen


    ' Last, see if we have to give in BIG points
    For I% = HEARTS To SPADES Step ONE_SUIT
        
        If I% <> Game_BriscolaSuit% Then
            c% = pl.Find(I% + 3)
            If c% <> CARD_NONE Then GoTo Chosen
        
            c% = pl.Find(I% + ACE)
            If c% <> CARD_NONE Then GoTo Chosen
        End If
    Next I%
    
    
    ' We should never get here undecided, however...
    c% = 0  ' 1st card


Chosen:
    
    If c% <> CARD_NONE Then
        
        ' animate card before removing
        pl.Selected(c%) = True
        Sleep 0.5
        pl.Selected(c%) = False

        Game_PlayCard = pl.RemoveCard(c%)
    Else
        Game_PlayCard = CARD_EMPTY
    End If
    
    Table_Enable

End Function

Function Game_PlayerName(pl%) As String
    
    If pl% = 1 Then
        Game_PlayerName = Game.Player1Name
    Else
        Game_PlayerName = Game.Player2Name
    End If

End Function

Sub Game_Save(fname As String)

    Dim S$
    On Error GoTo save_error

    Open fname For Random As #1
    
    On Error GoTo write_error
    
    S$ = Game.Player1
    Put #1, , S$
    S$ = Game.Player2
    Put #1, , S$
    S$ = Game.Stack1
    Put #1, , S$
    S$ = Game.Stack2
    Put #1, , S$
    S$ = Game.Deck
    Put #1, , S$
    S$ = Game.BRISCOLA
    Put #1, , S$
    S$ = Game.OnTable
    Put #1, , S$

    Put #1, , Hand_CardPlayer1
    Put #1, , Hand_CardPlayer2
    Put #1, , Hand_PlayerTurn
    Put #1, , Game_BriscolaSuit


    Game_FileName = fname
    Form_SetTitle Game
   

write_error:
    Close #1
    

save_error:
    Exit Sub
End Sub

Sub Game_SetDefaults()
    Player2_AutoPlay = True
    Game.Mode = MSG_MODE_NORMAL
    Game.MessageView = "Select [New] to start"
End Sub

Sub Game_ShowScore()
    
    Player1_Score% = Game_CalcScore(Game.Stack1)
    Player2_Score% = Game_CalcScore(Game.Stack2)
    
    Game.Score1 = Player1_Score%
    Game.Score2 = Player2_Score%

    Game.Score = "Score: " & Player1_Score% & ":" & Player2_Score%

End Sub

Sub Game_Start()
    Game_InProgress = True
    Game.GameTimer.Enabled = True
    Hand_Number = 1
    
    TraceClear

End Sub

Sub Hand_CheckCard1()
    
    If Hand_PlayerTurn = 1 Then
        Hand_CardPlayer1 = Game.OnTable.Card(0)
    Else
        Hand_CardPlayer2 = Game.OnTable.Card(0)
    End If

End Sub

Sub Hand_CheckWinner()

    Dim Suit1%, Suit2%, Val1%, Val2%

    Game_Msg "Checking Hand..."

    ' retrieve cards from scratch (in case we missed some DDE update)
    If Hand_PlayerTurn = 1 Then
        Hand_CardPlayer1 = Game.OnTable.Card(1)
        Hand_CardPlayer2 = Game.OnTable.Card(0)
    Else
        Hand_CardPlayer1 = Game.OnTable.Card(0)
        Hand_CardPlayer2 = Game.OnTable.Card(1)
    End If


    Suit1% = CardSuit(Hand_CardPlayer1)
    Suit2% = CardSuit(Hand_CardPlayer2)
    Val1% = CardValue(Hand_CardPlayer1)
    Val2% = CardValue(Hand_CardPlayer2)


    ' both have same suit?
    If Suit2% = Suit1% Then
        
        ' if so, higher card wins
        ' must first shift 3 and ace to higher values
        If Val1% = 3 Then Val1% = 14
        If Val2% = 3 Then Val2% = 14

        If Val1% = 1 Then Val1% = 15
        If Val2% = 1 Then Val2% = 15
        
        If Val1% > Val2% Then
            Hand_Winner = 1
        Else
            Hand_Winner = 2
        End If
    
    ' different suits
    Else
    
        ' check vs. briscola
        If Suit1% = Game_BriscolaSuit% Then
            Hand_Winner = 1
        
        ' player 1 doesn't have briscola
        Else
            
            ' if p2 has, he wins
            If Suit2% = Game_BriscolaSuit% Then
                Hand_Winner = 2
            
            ' otherwise, player who put card down 1st wins
            Else
                ' Hand_PlayerTurn indicates last player, so other was 1st
                If Hand_PlayerTurn = 1 Then
                    Hand_Winner = 2
                Else
                    Hand_Winner = 1
                End If
            End If
            
        End If
    End If

    

End Sub

Sub Hand_Clear()
    Hand_CardPlayer1 = CARD_NONE              ' Clear last hand
    Hand_CardPlayer2 = CARD_NONE
End Sub

Sub Hand_Deal(PrevWinner%)

    Dim Card1 As Integer, Card2 As Integer
    
    Table_Disable "Dealing..."

    Game.Player1.Action = CARDS_ACTION_PACK
    Game.Player2.Action = CARDS_ACTION_PACK
    
    ' pick cards from deck
    If Game.Deck.NumCards > 0 Then
        Card1 = Game.Deck.TopCard
        
        ' check if dealing last hand
        If Game.Deck.NumCards > 0 Then
            Card2 = Game.Deck.TopCard
        Else
            Card2 = Game.BRISCOLA.TopCard
        End If

        ' give cards according to last hand results
        If PrevWinner% = 1 Then
            Game.Player1.TopCard = Card1
            Sleep 0.2
            Game.Player2.TopCard = Card2
        Else
            Game.Player1.TopCard = Card2
            Sleep 0.2
            Game.Player2.TopCard = Card1
        End If

    End If

    If Game.OptPeek.Checked Then
        Game.Player2.Action = CARDS_ACTION_TURN_UP
    End If
    
    Table_Enable
    
    If App_Debug Then
        Trace "Hand_Deal:  " & CardDesc$(Card1) & ",  " & CardDesc$(Card2)
        If Game.Player1.NumCards <> Game.Player2.NumCards Then
            Trace "Hand_Deal: FAILED - players have different # of cards"
        End If
    End If

End Sub

'
'
Sub Hand_DealFirst()
    
    If Game.Deck.NumCards = 0 Then Exit Sub
    
    If App_Debug Then Trace "Hand_DealFirst:"

    Table_Disable "Dealing..."
    
    Game_NextPlayer

    ' player 2 deals, player 1 gets cards first
    
    If Hand_PlayerTurn = 1 Then
        Game.Player2.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player1.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player2.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player1.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player2.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player1.TopCard = Game.Deck.TopCard
    Else
        Game.Player1.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player2.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player1.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player2.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player1.TopCard = Game.Deck.TopCard
        Sleep 0.2
        Game.Player2.TopCard = Game.Deck.TopCard
    End If

    Game.BRISCOLA.TopCard = Game.Deck.TopCard
    
    If Game.OptPeek.Checked Then
        Game.Player2.Action = CARDS_ACTION_TURN_UP
    End If

    Game_NextPlayer
    Table_Enable

End Sub

Sub Hand_Next()
    
    Hand_Stash Hand_Winner    ' give cards to winner (OnTable calls Hand_Clear)
    Game_ShowScore            '
    
    ' note: we use Count() and not NumCards
    ' because cards may be unpacked (yet).
    If Game.Player1.Count(0) = 0 Then       ' is it over?
        Game_Finish False
    Else
        ' in network mode, only last player deals.
        ' other party will receive cards via DDE.
        ' This prevents conflicting update (cross DDE
        ' updates may come at different times...)
        If Game_Mode = MODE_NETWORK Then
            If Hand_PlayerTurn = 1 Then
                Hand_Deal Hand_Winner
            Else
                If App_Debug Then Trace "Hand_Next: network peer will deal"
            End If
        Else
            Hand_Deal Hand_Winner
        End If
        
        Game_NextPlayerIs Hand_Winner       ' Decide next player
        Hand_Number = Hand_Number + 1       ' Count hands
        
    End If
    
    Hand_Winner = 0                         ' Reset for next time
                                            ' GameTimer Will no longer call us

    If App_Debug Then Trace "Hand_Next: done. Next player is " & Hand_PlayerTurn

End Sub

Sub Hand_Stash(PrevWinner%)
    If App_Debug Then Trace "Hand_Stash"

    If PrevWinner% = 1 Then
        Game.Stack1.AddCards = Game.OnTable
    Else
        Game.Stack2.AddCards = Game.OnTable
    End If
    
    Game.OnTable.Clear

End Sub

Sub Options_Read(f As Form)
    Dim WorkDir$
    
    On Error Resume Next
    
    WorkDir$ = Profile_ReadString$(SEC_GLOBAL, KEY_WORKDIR, "")
    If WorkDir <> "" Then ChDir WorkDir
    
    RecentFile_Read f
    
    f.OptToolBar.Checked = Profile_ReadBool("Options", "ToolBar", True)
    f.OptStatusBar.Checked = Profile_ReadBool("Options", "StatusBar", True)
    f.mSound.Checked = Profile_ReadBool("Options", "Sound", True)


    f.ToolBar.Visible = f.OptToolBar.Checked
    f.StatusLine.Visible = f.OptStatusBar.Checked

End Sub

Sub Options_Write(f As Form)
    
    RecentFile_Write f
    Profile_WriteString SEC_GLOBAL, KEY_WORKDIR, CurDir$
    
    Profile_WriteBool "Options", "ToolBar", (f.OptToolBar.Checked)
    Profile_WriteBool "Options", "StatusBar", (f.OptStatusBar.Checked)
    Profile_WriteBool "Options", "Sound", (f.mSound.Checked)

End Sub

Sub Sleep(S As Single)

    Dim start
    
    start = Timer
    Do
        DoEvents
    Loop Until Timer > start + S
    
End Sub

Sub Table_Clear()
    
    Game.Stack1.NumCards = 0
    Game.Stack2.NumCards = 0
    Game.Player1.NumCards = 0
    Game.Player2.NumCards = 0
    Game.Deck.NumCards = 40
    Game.OnTable.NumCards = 0
    Game.BRISCOLA.NumCards = 0
    
    Game.Deck.Action = CARDS_ACTION_SHUFFLE

End Sub

Sub Table_Disable(Msg$)
    
    Game_Msg Msg$
    Screen.MousePointer = HOURGLASS
    
    ' Avoid nesting
    If Game.Enabled = True Then Game.Enabled = False
End Sub

Sub Table_Enable()
    
    Screen.MousePointer = Default
    
    ' Avoid nesting
    If Game.Enabled = False Then Game.Enabled = True
End Sub

Sub Trace(S$)
    
    If App_Debug Then
        TraceWin.LogBox.AddItem S$
        If TraceWin.LogBox.ListCount > 100 Then TraceWin.LogBox.RemoveItem 0
        TraceWin.LogBox.TopIndex = TraceWin.LogBox.ListCount - 1
    End If

End Sub

Sub TraceClear()
    TraceWin.LogBox.Clear
End Sub

Sub TraceMode(Mode%)
    If Mode% Then
        App_Debug = True
        TraceWin.Show
    Else
        App_Debug = False
        TraceWin.Hide
    End If
End Sub

